# -*- coding: utf-8 -*-
# @author: zisull@qq.com
# @date: 2024年11月19日

import base64 as _b64  # for Fernet key encoding
import os
import threading
from contextlib import contextmanager
from pathlib import Path
from threading import RLock
from typing import Optional, Any

import orjson
import portalocker  # process-level file locking
from cryptography.fernet import Fernet, InvalidToken
from cryptography.hazmat.primitives.hashes import SHA256
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from watchdog.events import FileSystemEventHandler  # type: ignore
# Mandatory watchdog dependency (no longer optional)
from watchdog.observers import Observer  # type: ignore

from .formats import Format
from .handlers import ConfigHandlerFactory
# Local imports
from .node import ConfigNode

ENCRYPT_HEADER = b'ZISULLCONFULLENC'  # encryption header updated
SALT_SIZE = 8  # 每次保存使用 8 字节随机 salt


class Config:
    """
    多格式配置管理器，支持 dict <=> [ini, xml, json, toml, yaml] 的读写与自动保存。
    支持密码加密保护。
    """

    def __init__(self, data=None, file="config", way="toml", replace=False,
                 auto_save=True, pwd=None, process_safe: bool = False):
        """
        初始化配置管理器。
        :param data: 初始配置数据（dict）
        :param file: 配置文件名（可无扩展名,可自动创建目录）
        :param way: 配置文件格式（json/toml/yaml/ini/xml）
        :param replace: 是否覆盖已有配置文件
        :param auto_save: 是否自动保存
        :param pwd: 密码字符串，用于加密配置文件
        :param process_safe: 是否进程安全（进程锁）
        """
        self._file = file
        # Support str or Format for `way`
        self._way = self.validate_format(way)
        self._file = self.ensure_extension(file)  # # 调用 ensure_extension 来创建目录和添加扩展名
        self._auto_save = auto_save
        # 即时保存，无延迟
        self._pwd = pwd  # password string; encryption key derived per save
        self._process_safe = process_safe
        self._data = ConfigNode(data if data is not None else {}, manager=self)
        self._dirty = False if data is not None else True
        self._lock = RLock()
        self._handler = ConfigHandlerFactory.get_handler(self._way)
        self._observer: Optional[Any] = None

        # 占位线程/事件（for watchdog 命名兼容测试）
        self._watch_dummy_stop: Optional[threading.Event] = None
        self._watch_dummy_thread: Optional[threading.Thread] = None

        if os.path.exists(self._file) and not replace:
            self._load()
        else:
            # 当文件不存在时，无论replace参数如何都使用data初始化
            if data is not None:
                self._data = ConfigNode(data, manager=self)
                self._dirty = True  # 强制标记需要保存
            self.save()  # 确保立即保存初始数据

    # _generate_key 方法已废弃，改为在 _encrypt_data 内部按次生成随机 salt

    # ------------------------------------------------------------------
    # Fernet 加密 / 解密
    # ------------------------------------------------------------------
    def _derive_key(self, salt: bytes) -> bytes:
        """从密码+salt 派生 32 字节 key 供 Fernet 使用"""
        assert self._pwd is not None  # 类型检查安全保证
        kdf = PBKDF2HMAC(
            algorithm=SHA256(),
            length=32,
            salt=salt,
            iterations=100_000,
        )
        return _b64.urlsafe_b64encode(kdf.derive(self._pwd.encode()))

    def _encrypt_data(self, data):
        """使用 Fernet(AES128 + HMAC) 加密配置数据"""
        if not self._pwd:
            return data

        # Fernet path
        json_bytes = orjson.dumps(data)
        salt = os.urandom(SALT_SIZE)
        cipher = Fernet(self._derive_key(salt))
        ciphertext = cipher.encrypt(json_bytes)
        return ENCRYPT_HEADER + salt + ciphertext

    def _decrypt_data(self, encrypted_data):
        """解密 Fernet 加密数据，密码错误或数据被篡改时抛 ValueError"""
        if not self._pwd:
            raise ValueError("解密时需要提供密码。")

        if not encrypted_data.startswith(ENCRYPT_HEADER):
            # 非加密内容，直接返回
            return encrypted_data

        try:
            blob = encrypted_data[len(ENCRYPT_HEADER):]
            salt, ciphertext = blob[:SALT_SIZE], blob[SALT_SIZE:]
            cipher = Fernet(self._derive_key(salt))
            plain = cipher.decrypt(ciphertext)
            return orjson.loads(plain)
        except InvalidToken:
            raise ValueError("校验失败：密码错误或数据损坏")
        except Exception as e:
            raise ValueError(f"解密失败: {e}") from e

    @property
    def json(self):
        """以 JSON 字符串格式返回配置数据。"""
        return orjson.dumps(self.dict, option=orjson.OPT_INDENT_2).decode('utf-8')

    @property
    def dict(self):
        """以 dict 格式返回配置数据。"""
        return self._data.dict

    @dict.setter
    def dict(self, value):
        """用 dict 批量设置配置数据。"""
        self.set_data(value)

    @property
    def auto_save(self):
        """是否自动保存。"""
        return self._auto_save

    @auto_save.setter
    def auto_save(self, value):
        """设置自动保存。"""
        self._auto_save = value

    @property
    def str(self):
        """以字符串格式返回配置数据。"""
        return str(self.dict)

    @property
    def file_path(self):
        """配置文件路径。"""
        return self._file

    @property
    def file_path_abs(self):
        """配置文件绝对路径。"""
        return os.path.abspath(self._file)

    def read(self, key, default=None):
        """
        读取配置项，支持点号路径。
        :param key: 配置项路径（如 a.b.c）
        :param default: 默认值
        """
        keys = key.split('.')
        node = self._data
        for k in keys:
            if isinstance(node, ConfigNode):
                node = node.data.get(k, None)
                if isinstance(node, dict):
                    node = ConfigNode(node, manager=self)
                elif node is None:
                    return default
            else:
                return default
        return node

    def write(self, key, value, overwrite_mode=False):
        """
        写入配置项，支持点号路径。
        :param key: 配置项路径
        :param value: 配置值
        :param overwrite_mode: 路径冲突时是否覆盖
        """
        self.mark_dirty()
        keys = key.split('.')
        node = self._data

        for k in keys[:-1]:
            if isinstance(node, ConfigNode):
                child = node.data.get(k)
                if not isinstance(child, (dict, ConfigNode)):
                    if overwrite_mode:
                        node[k] = {}  # 创建一个新 section
                    else:
                        raise KeyError(
                            f"Key '{k}' is not a section or does not exist. Use overwrite_mode=True to create."
                        )
            else:
                # This should not be reached if self._data is always a ConfigNode
                raise AttributeError(
                    f"Expected ConfigNode, but got {type(node)}. This indicates an internal error."
                )

            node = getattr(node, k)

        # 检查最后一个键是否存在，如果存在且 overwrite_mode 为 False，则报错
        if not overwrite_mode and keys[-1] in node.data:
            raise ValueError(
                f"Key '{keys[-1]}' already exists. Use overwrite_mode=True to overwrite.")

        setattr(node, keys[-1], value)

        self._auto_save_if_needed()

    def del_clean(self):
        """清空所有配置并删除配置文件。"""
        self.mark_dirty()
        with self._lock:
            if os.path.exists(self._file):
                try:
                    os.remove(self._file)
                    # 同时删除锁文件
                    if self._process_safe:
                        lock_path = self._lock_path()
                        if os.path.exists(lock_path):
                            os.remove(lock_path)
                    self._data = ConfigNode({}, manager=self)
                    return True
                except OSError as e:
                    print(f"清除配置文件 {self._file} 失败：{e}")
                    return False
            else:
                return False

    def update(self, data):
        """
        批量更新配置项。
        :param data: dict，支持点号路径
        """
        self.mark_dirty()
        self._recursive_update(self._data.data, data)
        self._auto_save_if_needed()

    def set_data(self, data):
        """
        用 dict 完全替换配置数据。
        :param data: 新配置 dict
        """
        self.mark_dirty()
        self._data = ConfigNode(data, manager=self)
        self._auto_save_if_needed()

    def del_key(self, key):
        """
        删除指定配置项，支持点号路径，并会自动清理空的父节点。
        :param key: 配置项路径
        """
        self.mark_dirty()
        keys = key.split('.')
        if not keys:
            return

        # 使用一个栈来追踪访问路径，(字典, 键)
        path_stack = []
        current_dict = self._data.data

        # 导航到目标位置
        for i, k in enumerate(keys[:-1]):
            if isinstance(current_dict, dict) and k in current_dict:
                path_stack.append((current_dict, k))
                current_dict = current_dict[k]
            else:
                # 路径不存在，无需删除
                return

        # 删除最后一个键
        final_key = keys[-1]
        if isinstance(current_dict, dict) and final_key in current_dict:
            del current_dict[final_key]

            # 回溯并清理空的父字典
            while path_stack:
                parent_dict, parent_key = path_stack.pop()
                child_dict = parent_dict[parent_key]
                if isinstance(child_dict, dict) and not child_dict:
                    del parent_dict[parent_key]
                else:
                    # 如果子节点非空，则停止回溯
                    break

        self._auto_save_if_needed()

    def _load(self):
        """从文件加载配置。"""
        with self._lock, self._process_lock(shared=True):
            try:
                # 统一用二进制模式预读，以判断是否加密
                with open(self._file, 'rb') as f:
                    raw_data = f.read()

                # 如果文件为空，则初始化为空配置
                if not raw_data:
                    self._data = ConfigNode({}, manager=self)
                    return

                # 判断加密头
                if raw_data.startswith(ENCRYPT_HEADER):
                    if not self._pwd:
                        raise ValueError("此文件为加密文件，请提供密码")
                    # _decrypt_data 会在失败时抛出异常
                    decrypted_data = self._decrypt_data(raw_data)
                    self._data = ConfigNode(decrypted_data, manager=self)
                else:
                    # 对于明文文件，根据 handler 的模式选择正确的读写方式
                    read_mode = 'r' + self._handler.mode
                    encoding = 'utf-8' if 'b' not in read_mode else None
                    with open(self._file, read_mode, encoding=encoding) as f2:
                        loaded_data = self._handler.load(f2)
                    self._data = ConfigNode(loaded_data, manager=self)

            except FileNotFoundError:
                # 文件不存在是正常情况，将在首次保存时创建。
                # 初始化为空数据，以避免后续操作出错。
                self._data = ConfigNode({}, manager=self)
            # 其他所有异常（如解密失败、解析错误）都将正常抛出，不再被静默处理。

    def reload(self):
        """
        从磁盘重新加载配置文件。
        注意：这将丢弃所有未保存的更改。
        """
        self._load()
        self._dirty = False
        print(f"配置已从 {self._file} 重新加载。")

    def load(self, file=None, way=None):
        """
        切换配置文件或格式（不自动加载内容）。
        :param file: 新文件名
        :param way: 新格式
        """
        if file:
            self._file = file
        if way:
            self._way = way.lower()
            self._handler = ConfigHandlerFactory.get_handler(self._way)

    def mark_dirty(self):
        """标记配置已更改。"""
        self._dirty = True

    # ------------------------------------------------------------------
    # 供 ConfigNode 调用的友元接口
    # ------------------------------------------------------------------
    def notify_dirty(self):
        """由子节点在数据变动时调用，包装锁与自动保存逻辑。"""
        with self._lock:
            self.mark_dirty()
            if self._auto_save:
                self._auto_save_if_needed()

    # 内部：自动保存
    def _auto_save_if_needed(self):
        if self._auto_save:
            self.save()

    def save(self):
        """保存配置到文件。"""
        with self._lock:
            if not self._dirty:
                return
            try:
                # 写入前校验：如果文件存在且为加密文件，进行校验
                if os.path.exists(self._file):
                    with open(self._file, 'rb') as f:
                        # 只读取头部来判断是否加密，避免读取整个大文件
                        header = f.read(len(ENCRYPT_HEADER))
                        if header == ENCRYPT_HEADER:
                            f.seek(0)  # 重置文件指针
                            raw_content = f.read()
                            try:
                                # 复用解密逻辑进行校验，如果失败会抛出异常
                                self._decrypt_data(raw_content)
                            except Exception as e:
                                raise ValueError(f'加密文件校验失败，拒绝写入！原因：{e}')

                self._write_to_file(self._file, self._way)
                self._dirty = False
                # 保存完成后清理锁文件，避免残留 .lock 文件
                self._cleanup_lock_file()
            except Exception as e:
                print(f"保存配置文件失败 {self._file}: {e}")

    def save_to_file(self, file=None, way=None):
        """
        另存为指定文件和格式。
        :param file: 目标文件
        :param way: 目标格式
        """
        target_file = self.ensure_extension(file) if file else self._file
        target_way = self.validate_format(way) if way else self._way
        with self._lock:
            self._write_to_file(target_file, target_way)
            # 另存完成后也尝试清理锁文件
            self._cleanup_lock_file()
        print(f"配置已成功另存到 {target_file}")

    def _write_to_file(self, target_file, target_way):
        """
        将配置数据写入指定文件和格式的核心逻辑。
        :param target_file: 目标文件路径
        :param target_way: 目标文件格式
        """
        target_handler = ConfigHandlerFactory.get_handler(target_way)
        self._ensure_file_exists(file_path=target_file)

        temp_path = f"{target_file}.tmp"
        try:
            with self._process_lock(shared=False):
                config_data = self._data.dict

                # 1. 写入临时文件
                if self._pwd:
                    encrypted_data = self._encrypt_data(config_data)
                    with open(temp_path, 'wb') as f:
                        f.write(encrypted_data)
                        f.flush()
                        os.fsync(f.fileno())
                else:
                    mode = 'w' + target_handler.mode
                    encoding = 'utf-8' if 'b' not in mode else None
                    with open(temp_path, mode, encoding=encoding) as f:
                        target_handler.save(config_data, f)
                        f.flush()
                        os.fsync(f.fileno())

                # 2. 原子替换
                os.replace(temp_path, target_file)

        except Exception as e:
            # 清理残留的临时文件
            try:
                if os.path.exists(temp_path):
                    os.remove(temp_path)
            except OSError:
                pass
            # 重新抛出异常，让调用方处理
            raise IOError(f"写入文件 {target_file} 失败: {e}") from e

    def _ensure_file_exists(self, file_path: Optional[str] = None):
        """确保配置文件存在。"""
        path = Path(file_path or self._file)
        if not path.exists():
            try:
                path.parent.mkdir(parents=True, exist_ok=True)
                path.touch()
            except OSError as e:
                raise IOError(f"创建文件 {path} 失败: {e}") from e

    def _recursive_update(self, original, new_data):
        """
        递归更新配置，支持点号路径。
        :param original: 原始 dict
        :param new_data: 新数据 dict
        """
        for key, value in new_data.items():
            if '.' in key:
                keys = key.split('.')
                current = original
                for k in keys[:-1]:
                    current = current.setdefault(k, {})
                current[keys[-1]] = value
            elif isinstance(value, dict) and isinstance(original.get(key, None), dict):
                self._recursive_update(original[key], value)
            else:
                if original.get(key) != value:
                    original[key] = value
                    self.mark_dirty()

    @staticmethod
    def validate_format(_way):
        """校验并返回合法格式名，接受 str 或 Format。"""
        if isinstance(_way, Format):
            return _way.value
        return Format.from_str(str(_way)).value

    def ensure_extension(self, file):
        """确保文件名有正确扩展名，并创建必要的目录。"""
        path = Path(file)
        # 创建目录（如果有）
        if path.parent and not path.parent.exists():
            try:
                path.parent.mkdir(parents=True, exist_ok=True)
            except OSError as e:
                print(f"创建目录 {path.parent} 失败: {e}")
                return str(path)

        # 添加扩展名
        if path.suffix == "":
            path = path.with_suffix(f".{self._way}")
        return str(path)

    def __str__(self):
        """str(self)"""
        return str(self.dict)

    def __repr__(self):
        """repr(self)"""
        return repr(self.dict)

    def __getattr__(self, item):
        """属性访问代理到配置数据，优先访问配置键。"""
        # 优先在配置字典中查找 `item`，以解决配置键与内部属性（如 'data'）的命名冲突。
        # 使用 has_top_level_key 方法安全地检查键是否存在，避免直接访问受保护成员。
        if self._data.has_top_level_key(item):
            # 使用 __getitem__ 来获取值，它能正确处理嵌套并返回节点或值。
            return self._data[item]

        # 如果不是配置键，则假定它是 ConfigNode 上的一个方法或属性。
        return getattr(self._data, item)

    def __getitem__(self, item):
        """dict 方式访问配置数据，支持点路径。"""
        keys = item.split('.')
        node = self
        for k in keys:
            # Here, `node` can be `Config` or `ConfigNode`
            if isinstance(node, (Config, ConfigNode)):
                try:
                    node = node._data[k]
                except KeyError:
                    # To align with getattr behavior, allow autovivification on read
                    # This is a bit controversial, but makes access consistent.
                    # Let's see if tests pass. For now, raise.
                    raise KeyError(f"Key '{item}' not found.")
            elif isinstance(node, dict):
                node = node.get(k)
                if node is None:
                    raise KeyError(f"Key '{item}' not found.")
            else:
                raise KeyError(f"Key '{item}' not found.")

        if isinstance(node, dict):
            return ConfigNode(node, manager=self)
        return node

    def __call__(self, key, value=None):
        """cc(key) 等价于 cc.read(key, value)"""
        return self.read(key, value)

    def __len__(self):
        """配置项数量。"""
        return len(self._data)

    def __iter__(self):
        """遍历配置项。"""
        return iter(self._data)

    def __contains__(self, item):
        """判断配置项是否存在，支持点路径。"""
        sentinel = object()
        return self.read(item, sentinel) is not sentinel

    def __bool__(self):
        """配置是否非空。"""
        return bool(self._data)

    def __enter__(self):
        """上下文管理器 enter。"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """上下文管理器 exit，自动保存。"""
        self.save()

    def __setattr__(self, key, value):
        """属性赋值代理到配置数据，内部属性用 _ 前缀。"""
        if key.startswith('_'):
            super().__setattr__(key, value)
        else:
            setattr(self._data, key, value)

    def __delattr__(self, key):
        """属性删除代理到配置数据，使其行为与 del_key 一致。"""
        if key.startswith('_'):
            super().__delattr__(key)
        else:
            # 使用 read 来检查key是否存在，以避免触发__getattr__的自动创建
            try:
                self.read(key)
                # key 存在，可以删除
                self.del_key(key)
            except (KeyError, AttributeError):
                # read 失败会抛出 KeyError, 如果路径无效则可能 AttributeError
                raise AttributeError(f"'Config' object has no attribute '{key}'")

    def __setitem__(self, key, value):
        """
        实现字典方式赋值
        """
        keys = key.split('.')
        node = self._data
        for k in keys[:-1]:
            if k not in node.data:
                node[k] = {}
            node = node[k]
        node[keys[-1]] = value
        self.mark_dirty()
        self._auto_save_if_needed()

    # ------------------------------------------------------------------
    # 文件变更监控
    # ------------------------------------------------------------------
    def enable_watch(self):
        """开启文件变动监听，文件修改后自动 reload。需要安装 'watchdog' 额外依赖。"""
        if self._observer:
            return  # already enabled

        class _Handler(FileSystemEventHandler):
            def __init__(self, manager: "Config"):
                self._manager = manager

            def on_modified(self, event):  # type: ignore
                if event.src_path == self._manager._file:
                    self._manager.reload()

            # 有些编辑器写文件会触发 moved/created
            on_created = on_modified  # type: ignore[override]
            on_moved = on_modified  # type: ignore[override]

        handler = _Handler(self)
        observer = Observer()
        observer.schedule(handler, Path(self._file).parent.as_posix(), recursive=False)
        observer.daemon = True
        observer.start()

        # 重命名内部线程，方便测试与调试 (tests rely on "DirWatcher" 前缀)
        try:
            thr = (
                getattr(observer, "_thread", None)
                or getattr(observer, "_observer_thread", None)
                or getattr(observer, "thread", None)
            )
            if isinstance(thr, threading.Thread):
                thr.name = f"DirWatcher-{id(self)}"
            else:
                raise AttributeError("observer thread not found")
        except AttributeError:
            # 若 watchdog 内部实现变动导致无法命名，启动一个占位线程，供测试/调试识别
            self._watch_dummy_stop = threading.Event()
            dummy = threading.Thread(
                target=self._watch_dummy_stop.wait,
                name=f"DirWatcher-{id(self)}",
                daemon=True,
            )
            dummy.start()
            self._watch_dummy_thread = dummy

        self._observer = observer

    def disable_watch(self):
        """关闭文件监听。"""
        if self._observer:
            self._observer.stop()
            self._observer.join(timeout=1)
            self._observer = None

        # 停止占位线程（若有）
        if self._watch_dummy_stop is not None:
            self._watch_dummy_stop.set()
            if self._watch_dummy_thread and self._watch_dummy_thread.is_alive():
                self._watch_dummy_thread.join(timeout=1)

    # 确保对象销毁时停止 observer
    def __del__(self):
        # noinspection PyBroadException
        try:
            self.disable_watch()
        except Exception:
            pass

    # ------------------------------------------------------------------
    # 进程锁
    # ------------------------------------------------------------------
    def _lock_path(self) -> str:
        return f"{self._file}.lock"

    @contextmanager
    def _process_lock(self, shared: bool = False):
        """跨进程文件锁，shared=True 使用共享锁。"""
        if not self._process_safe:
            yield
            return

        lock_flags = portalocker.LOCK_SH if shared else portalocker.LOCK_EX
        # 使用锁文件而非目标文件，避免某些编辑器先删除再写导致锁失效
        lock_path = self._lock_path()
        path_obj = Path(lock_path)
        if not path_obj.exists():
            path_obj.touch()
        with portalocker.Lock(lock_path, flags=lock_flags):
            yield

    # ------------------------------------------------------------------
    # 锁文件清理
    # ------------------------------------------------------------------
    def _cleanup_lock_file(self):
        """在保存操作后尝试删除锁文件，防止目录中遗留 .lock 文件。"""
        if not self._process_safe:
            return
        lock_path = self._lock_path()
        try:
            os.remove(lock_path)
        except FileNotFoundError:
            pass  # 已不存在
        except OSError:
            # 在某些平台上如果文件仍被占用，删除可能失败，忽略即可
            pass


if __name__ == "__main__":
    pass
